#ifndef HAL_ZBM_H
#define HAL_ZBM_H


/*******************************************************************************************************
 **************************                   I/O Ʈ                    **************************
 *******************************************************************************************************/


//-------------------------------------------------------------------------------------------------------
// Port A
//-------------------------------------------------------------------------------------------------------
//#define               0 // PA.0 - 밡 
//#define               1 // PA.1 - 밡 
//#define               2 // PA.2 - 밡 
//#define               3 // PA.3 - 밡 
//#define               4 // PA.4 - 밡 
//#define               5 // PA.5 - 밡 
//#define               6 // PA.6 - 밡 
//#define               7 // PA.7 - 밡 
//-------------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------------
// Port B
//-------------------------------------------------------------------------------------------------------
#define CSN             0 // PB.0 - Output: SPI Chip Select (CS_N)
#define SCK             1 // PB.1 - Output: SPI Serial Clock (SCLK)
#define MOSI            2 // PB.2 - Output: SPI Master out - slave in (MOSI)
#define MISO            3 // PB.3 - Input:  SPI Master in - slave out (MISO)
//#define               4  // PB.4 - 밡 
//#define               5  // PB.5 - 밡 
//#define               6  // PB.6 - 밡 
//#define               7  // PB.7 - 밡 
//-------------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------------
// Port C
//-------------------------------------------------------------------------------------------------------
//#define               0 // PC.0 - 밡 
//#define               1 // PC.1 - 밡 
//#define               2 // PC.2 - 밡 
//#define               3 // PC.3 - 밡 
//#define               4 // PC.4 - 밡 
//#define               5 // PC.5 - 밡 
//#define               6 // PC.6 - 밡 
//#define               7 // PC.7 - 밡 
//-------------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------------
// Port D
//-------------------------------------------------------------------------------------------------------
#define FIFOP           0 // PD.0 - Input:  FIFOP from CC2420
#define FIFO            1 // PD.1 - Input:  FIFO from CC2420
#define UART1_RXD       2 // PD.2 - Input:  UART1 RXD
#define UART1_TXD       3 // PD.3 - Output: UART1 TXD
#define SFD             4 // PD.4 - Input:  SFD from CC2420
#define UART1_RTS       5 // PD.5 - Output: UART HW handshaking: RTS
#define CCA             6 // PD.6 - Input:  CCA from CC2420
#define UART1_CTS       7 // PD.7 - Input:  UART HW handshaking: CTS
//-------------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------------
// Port E
//-------------------------------------------------------------------------------------------------------
//#define               0  // PE.0 - 밡 
//#define               1  // PE.1 - 밡 
//#define               2  // PE.2 - 밡 
//#define               3  // PE.3 - 밡 
//#define               4  // PE.4 - 밡 
//#define               5  // PE.5 - 밡 
//#define               6  // PE.6 - 밡 
//#define               7  // PE.7 - 밡 
//-------------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------------
// Port F
//-------------------------------------------------------------------------------------------------------
#define ADC0            0 // PF.0 - Input: з¼ Է
//#define               1 // PF.1 - 밡 
//#define               2 // PF.2 - 밡 
//#define               3 // PF.3 - 밡 
//-------------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------------
// Port G
//-------------------------------------------------------------------------------------------------------
//#define               0 // PG.0 - 밡 
//#define               1 // PG.1 - 밡 
//#define               2 // PG.2 - 밡 
#define VREG_EN         3 // PG.3 - Output: VREG_EN to CC2420
#define RESET_N         4 // PG.4 - Output: RESET_N to CC2420
//-------------------------------------------------------------------------------------------------------


/*******************************************************************************************************
 **************************                 ʱȭ ũ                 **************************
 *******************************************************************************************************/


//-------------------------------------------------------------------------------------------------------
// I/O Ʈ ʱȭ ũ 
//-------------------------------------------------------------------------------------------------------
#define PORT_INIT() \
    do { \
        SFIOR |= BM(PUD); \
        DDRB  = BM(MOSI) | BM(SCK) | BM(CSN); \
        PORTB = BM(MOSI) | BM(SCK) | BM(CSN); \
        DDRD  = BM(UART1_RTS); \
        PORTD = BM(UART1_RTS) | BM(UART1_CTS);\
        DDRG  = BM(RESET_N) | BM(VREG_EN); \
        PORTG = BM(RESET_N); \
    } while (0)
//-------------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------------
// ܺ SRAM ʱȭ ũ 
//-------------------------------------------------------------------------------------------------------
#define ENABLE_EXT_RAM() (MCUCR |= BM(SRE))
//-------------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------------
// SPI  ũ 
//-------------------------------------------------------------------------------------------------------
#define SPI_ENABLE()                (PORTB &= ~BM(CSN))
#define SPI_DISABLE()               (PORTB |= BM(CSN))
//-------------------------------------------------------------------------------------------------------


/*******************************************************************************************************
 **************************                 CC2420 PIN ACCESS                 **************************
 *******************************************************************************************************/


//-------------------------------------------------------------------------------------------------------
// Pin status
//-------------------------------------------------------------------------------------------------------
#define FIFO_IS_1       (!!(PIND & BM(FIFO)))
#define CCA_IS_1        (!!(PIND & BM(CCA)))
#define RESET_IS_1      (!!(PING & BM(RESET_N)))
#define VREG_IS_1       (!!(PING & BM(VREG_EN)))
#define FIFOP_IS_1      (!!(PIND & BM(FIFOP)))
#define SFD_IS_1        (!!(PIND & BM(SFD)))
//-------------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------------
// The CC2420 reset pin
//-------------------------------------------------------------------------------------------------------
#define SET_RESET_ACTIVE()    PORTG &= ~BM(RESET_N)
#define SET_RESET_INACTIVE()  PORTG |= BM(RESET_N)
//-------------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------------
// CC2420 voltage regulator enable pin
//-------------------------------------------------------------------------------------------------------
#define SET_VREG_ACTIVE()     PORTG |= BM(VREG_EN)
#define SET_VREG_INACTIVE()   PORTG &= ~BM(VREG_EN)
//-------------------------------------------------------------------------------------------------------


/*******************************************************************************************************
 **************************                   SERIAL PORTS                    **************************
 *******************************************************************************************************/


//-------------------------------------------------------------------------------------------------------
// UART1 hardware flow control
//-------------------------------------------------------------------------------------------------------
#define CLR_RTS()                  (PORTD &= ~BM(UART1_RTS))
#define SET_RTS()                  (PORTD |= BM(UART1_RTS))

#define SET_CTS()                  (PORTD |= BM(UART1_CTS))
#define CLR_CTS()                  (PORTD &= ~BM(UART1_CTS))
//-------------------------------------------------------------------------------------------------------


/*******************************************************************************************************
 *******************************************************************************************************
 **************************               EXTERNAL INTERRUPTS                 **************************
 *******************************************************************************************************
 *******************************************************************************************************/


//-------------------------------------------------------------------------------------------------------
// Rising edge trigger for external interrupt 0 (FIFOP)
//-------------------------------------------------------------------------------------------------------
#define FIFOP_INT_INIT()            do { EICRA |= 0x03; CLEAR_FIFOP_INT(); } while (0)
//-------------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------------
// FIFOP on external interrupt 0
//-------------------------------------------------------------------------------------------------------
#define ENABLE_FIFOP_INT()          do { EIMSK |= 0x01; } while (0)
#define DISABLE_FIFOP_INT()         do { EIMSK &= ~0x01; } while (0)
#define CLEAR_FIFOP_INT()           do { EIFR = 0x01; } while (0)
//-------------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------------
// SFD interrupt on timer 1 capture pin
//-------------------------------------------------------------------------------------------------------
#define ENABLE_SFD_CAPTURE_INT()    do { TIMSK |= BM(TICIE1); } while (0)
#define DISABLE_SFD_CAPTURE_INT()   do { TIMSK &= ~BM(TICIE1); } while (0)
#define CLEAR_SFD_CAPTURE_INT()     do { TIFR = BM(ICF1); } while (0)
//-------------------------------------------------------------------------------------------------------


/*******************************************************************************************************
 **************************               APPLICATION DEBUGGING               **************************
 *******************************************************************************************************/


//-------------------------------------------------------------------------------------------------------
// Controlled application crash (flashes the LEDs forever to indicate an error code)
//-------------------------------------------------------------------------------------------------------
#define EXCEPTION(n) \
    do { \
        DISABLE_GLOBAL_INT(); \
        SET_LED_MASK(n); \
        halWait(50000); \
        SET_LED_MASK(0); \
        halWait(50000); \
    } while (TRUE)
//-------------------------------------------------------------------------------------------------------


/*******************************************************************************************************
 **************************       ߰ ũ - ܺġ        **************************
 *******************************************************************************************************/


//-------------------------------------------------------------------------------------------------------
// LCD,  ܺġ  ʱȭ   ũ
//-------------------------------------------------------------------------------------------------------



// ⿡ ߰



//-------------------------------------------------------------------------------------------------------




//-------------------------------------------------------------------------------------------------------
#endif
